/* cssbj32.c */
/*
   Solve the system of eqautions
   y = a*x*(1 - x*x)  and x = b*(y - 2/5)*(1 - y*y) + c
*/
#include "../csc/csc.h"
#include "../eagle/eagle.h"
#include "Newton.h"
#define  NN 100

void print_data(x, y, r, conv, w0, w1, w2)
    double x, y, r, conv; Window w0, w1, w2;
{
    unsigned long black;
    black = set_color(0, 0, 0); set_foreground(black);
    w_select(w1); w_clear();
    x_printf(10, 30, "x = %lf", x); x_printf(10, 70, "y = %lf", y);
    w_select(w2); w_clear();
    x_printf(10, 30, "r = %lg", r); x_printf(10, 70, "conv = %lg", conv);
    w_select(w0);
}

main()
{
    Window w0, w1, w2, w; KeySym key;
    int wx, wy, BP, KP, button, etype, xint, yint;
    unsigned long blue, red;
    unsigned long SpringGreen, SkyBlue1, yellow, chocolate3, ivory2,
                  c1, c2, c3, c4, c5;
    double xx1[NN], yy1[NN], xx2[NN], yy2[NN]; double dm;
    double x[2]; double r, conv, xmin, xmax, ymin, ymax, x0, y0;
    int i, Ilimit;
    extern void print_data();

    a = 3.0; b = 4.0; c = 1.0; conv = 1.0e-9; Ilimit = 30;

    xmin = -2.0; xmax = 2.0; ymin = -2.0; ymax = 2.0;
    wx = 620; wy = 600;
    w0 = x_init(wx, wy+100, "Eagle_data");
    w1 = w_open(w0,  20, wy+10, 270, 80);
    w2 = w_open(w0, 320, wy+10, 270, 80);
    blue = set_color(0, 0, 255); red  = set_color(255, 0, 0);
    SpringGreen = set_color(  0, 255, 127);
    SkyBlue1    = set_color(135, 206, 255);
    yellow      = set_color(255, 255,   0);
    chocolate3  = set_color(205, 102,  29);
    ivory2      = set_color(238, 238, 224);
    c1          = set_color(123, 34,   23);
    c2          = set_color( 67, 149,   0);
    c3          = set_color(255,   0, 234);
    c4          = set_color(  0,   0,  36);
    c5          = set_color(255,   0,  25);
    m_trans_def(xmin, ymin, xmax, ymax, 20, 20, wx-40, wy-20);
    dm = 4.0/Ax_eagle;
    m_line(xmin, 0.0, xmax, 0.0); m_line(0.0, ymin, 0.0, ymax);
    for (i = 0; i < NN; i++) {
       xx1[i] = xmin + (xmax-xmin)*i/(NN-1);
       yy2[i] = ymin + (ymax-ymin)*i/(NN-1);
       yy1[i] = a*xx1[i]*(1.0 - xx1[i]*xx1[i]);
       xx2[i] = b*(yy2[i] - 2.0/5.0)*(1.0 - yy2[i]*yy2[i]) + c;
    }
    m_lines(NN, xx1, yy1); m_lines(NN, xx2, yy2);

    KP = 1; BP = 1;
    while (1) {
       etype = e_wait(KP, BP, 0, 0, 0, 0, &xint, &yint, &button, &w, &key);
       if ((etype==KeyPress)&&(key == XK_Escape)) break;
       if (etype==ButtonPress) {
          m_trans_x_to_m(xint, yint, &x[0], &x[1]); x0 = x[0]; y0 = x[1];
          set_foreground(blue); m_circle(x[0], x[1], dm, dm, 1);
          r = 1.0; i = 0;
	  print_data(x[0], x[1], r, conv, w0, w1, w2); set_foreground(blue);
	  while (r > conv) {
             r = Newton(2, x); m_circle(x[0], x[1], dm, dm, 1);
             print_data(x[0], x[1], r, conv, w0, w1, w2); set_foreground(blue);
	     i = i + 1; if (i > Ilimit) break;
	  }
          set_foreground(red); m_circle(x[0], x[1], 2.0*dm, 2.0*dm, 1);
	  w_select(w2); w_clear(); 
	  if (r < conv)
	  {  set_foreground(blue); x_printf(20, 50, "Converged"); }
	  if ((i > Ilimit) && (r > conv))
	  {  set_foreground(red);  x_printf(20, 50, "Not Converged"); }
	  w_select(w0);
	  if ((i > Ilimit) && (r > conv)) set_foreground(ivory2);
	  if ((r<conv)&&(x[0]<0.0)&&(x[1]>0.0))  set_foreground(SpringGreen);
	  if ((r<conv)&&(x[0]>0.0)&&(x[0]<0.5)&&(x[1]>0.5)) set_foreground(SkyBlue1);
	  if ((r<conv)&&(x[0]>0.5)&&(x[1]>0.5)) set_foreground(yellow);
	  if ((r<conv)&&(x[0]>0.5)&&(x[1]>0.0)&&(x[1]<0.5)) set_foreground(chocolate3);
          if ((r<conv)&&(x[0]>0.0)&&(x[0]<0.5)&&(x[1]>0.0)&&(x[1]<0.5)) set_foreground(c1);
          if ((r<conv)&&(x[0]>0.0)&&(x[1]<0.0)) set_foreground(c2);
          if ((r<conv)&&(x[0]<-0.5)&&(x[1]>-0.5)&&(x[1]<0.0)) set_foreground(c3);
          if ((r<conv)&&(x[0]<-0.5)&&(x[1]<-0.5)) set_foreground(c4);
          if ((r<conv)&&(x[0]>-0.5)&&(x[0]<0.0)&&(x[1]<-0.5)) set_foreground(c5);
	  x[0] = x0; x[1] = y0; r = 1.0; i =0; m_circle(x[0], x[1], dm, dm, 1);
	  while (r > conv) {
	     r = Newton(2, x); m_circle(x[0], x[1], dm, dm, 1);
	     i = i + 1; if (i > Ilimit) break;
	  }
          m_circle(x[0], x[1], 2.0*dm, 2.0*dm, 1);
       }
    }
    w_close(w1); w_close(w2); w_close(w0); x_term();
}

