/* cssbj31.c */
/*
   Solve the system of eqautions
   y = a*x*(1-x*x)  and x = b*(y - 4/5)*(1 - y*y)
*/
#include "../csc/csc.h"
#include "../eagle/eagle.h"
#include "Newton.h"
#define  NN 100

void print_data(x, y, r, conv, w0, w1, w2)
    double x, y, r, conv; Window w0, w1, w2;
{
    unsigned long black;
    black = set_color(0, 0, 0); set_foreground(black);
    w_select(w1); w_clear();
    x_printf(10, 30, "x = %lf", x); x_printf(10, 70, "y = %lf", y);
    w_select(w2); w_clear();
    x_printf(10, 30, "r = %lg", r); x_printf(10, 70, "conv = %lg", conv);
    w_select(w0);
}

main()
{
    Window w0, w1, w2, w3, w; KeySym key;
    int wx, wy, BP, KP, button, etype, xint, yint;
    unsigned long blue, red, black;
    double xx1[NN], yy1[NN], xx2[NN], yy2[NN]; double dm;
    double x[2]; double r, conv, xmin, xmax, ymin, ymax;
    int i, Ilimit;
    extern void print_data();

    a = 3.0; b = 4.0; c = 1.0; conv = 1.0e-9; Ilimit = 30;

    xmin = -2.0; xmax = 2.0; ymin = -2.0; ymax = 2.0;
    wx = 620; wy = 600;
    w0 = x_init(wx, wy+100, "Eagle_data");
    w1 = w_open(w0,  20, wy+10, 270, 80);
    w2 = w_open(w0, 320, wy+10, 270, 80);
    w3 = w_open(w0,   1,     1,  50, 15);
    blue = set_color(0, 0, 255); red  = set_color(255, 0, 0);
    black = set_color(0, 0, 0);
    m_trans_def(xmin, ymin, xmax, ymax, 20, 20, wx-40, wy-20);
    dm = 2.0/Ax_eagle;
    m_line(xmin, 0.0, xmax, 0.0); m_line(0.0, ymin, 0.0, ymax);
    for (i = 0; i < NN; i++) {
       xx1[i] = xmin + (xmax-xmin)*i/(NN-1);
       yy2[i] = ymin + (ymax-ymin)*i/(NN-1);
       yy1[i] = a*xx1[i]*(1.0 - xx1[i]*xx1[i]);
       xx2[i] = b*(yy2[i] - 2.0/5.0)*(1.0 - yy2[i]*yy2[i]) + c;
    }
    m_lines(NN, xx1, yy1); m_lines(NN, xx2, yy2);
    w_select(w3); x_printf(0, 10, "clear"); w_select(w0);

    KP = 1; BP = 1;
    while (1) {
       etype = e_wait(KP, BP, 0, 0, 0, 0, &xint, &yint, &button, &w, &key);
       if ((etype==KeyPress)&&(key == XK_Escape)) break;
       if ((w==w3)&&(etype==ButtonPress)) {
	  set_foreground(black); w_clear();
	  m_line(xmin, 0.0, xmax, 0.0); m_line(0.0, ymin, 0.0, ymax);
	  m_lines(NN, xx1, yy1); m_lines(NN, xx2, yy2);
       }
       if ((w==w0)&&(etype==ButtonPress)) {
          m_trans_x_to_m(xint, yint, &x[0], &x[1]);
          set_foreground(blue); m_circle(x[0], x[1], dm, dm, 1);
          r = 1.0; i = 0;
	  print_data(x[0], x[1], r, conv, w0, w1, w2); set_foreground(blue);
	  while (button == 2) {
	     r = Newton(2, x); m_circle(x[0], x[1], dm, dm, 1);
             print_data(x[0], x[1], r, conv, w0, w1, w2); set_foreground(blue);
	     if (r <= conv) break;
	     i = i + 1; if (i > Ilimit) break;
	     etype = e_wait(0, BP, 0, 0, 0, 0, &xint, &yint, &button, &w, &key);
          }
	  while (r > conv) {
             r = Newton(2, x); m_circle(x[0], x[1], dm, dm, 1);
             print_data(x[0], x[1], r, conv, w0, w1, w2); set_foreground(blue);
	     i = i + 1; if (i > Ilimit) break;
	  }
          set_foreground(red); m_circle(x[0], x[1], 2.0*dm, 2.0*dm, 1);
	  w_select(w2); w_clear(); 
	  if (r < conv)
	  {  set_foreground(blue); x_printf(20, 50, "Converged"); }
	  if ((i > Ilimit) && (r > conv))
	  {  set_foreground(red);  x_printf(20, 50, "Not Converged"); }
	  w_select(w0);
       }
    }
    w_close(w1); w_close(w2); w_close(w0); x_term();
}

