#include <stdio.h>
#include <glsc.h>
#include <math.h>

#define GRAPH_NAME "graph12"

#define WINDOW_X_SIZE 120.0
#define WINDOW_Y_SIZE 140.0

#define SIGN "12 / t930028 / Nobuyuki Okada"
#define SIGN_LEFT_MM 5.0
#define SIGN_TOP_MM 20.0

#define X_LEFT_MM 10.0
#define Y_TOP_MM 30.0
#define X_SIZE_MM 100.0
#define Y_SIZE_MM 100.0 

#define N_X 41 
#define N_Y 41
#define N_CONTOUR_LINE 10

#define X_MAX 10.0
#define X_MIN 0.0
#define Y_MAX 10.0
#define Y_MIN 0.0

#define V_FACTOR 0.5

extern void graphic_initialize();
extern double func();
extern void g_vector();
extern double grad_x();
extern double grad_y();

main() {
   int i, j;
   double del_x, del_y, x, y;
   double mountain[N_X][N_Y], summit, foot, del_h, height;
   double v_x, v_y, v_len;

   graphic_initialize();

   del_x = (X_MAX - X_MIN) / (N_Y - 1);
   del_y = (Y_MAX - Y_MIN) / (N_Y - 1);

   for (i = 0; i < N_X; i++) {
      x = X_MIN + i*del_x;
   for (j = 0; j < N_Y; j++) {
      y = Y_MIN + j*del_y;
      mountain[i][j] = func(x, y);
   }}

   summit = foot = mountain[0][0];

   for (i = 0; i < N_X; i++) {
   for (j = 0; j < N_Y; j++) {
      if (summit < mountain[i][j]) summit = mountain[i][j];
      else if (foot > mountain[i][j]) foot = mountain[i][j];
   }}

   del_h = (summit - foot) / N_CONTOUR_LINE;
   
   for (i = 0; i <= N_CONTOUR_LINE; i++) {
      height = foot + i*del_h;
      g_contln(X_MIN, X_MAX, Y_MIN, Y_MAX, mountain, N_X, N_Y, height);
   }

   g_marker_size( 1 );
   g_marker_type(G_MARKER_F_CIRC);

   for (i = 0; i < N_X; i = i + 4) {
   for (j = 0; j < N_Y; j = j + 4) {
      x = i*del_x;
      y = j*del_y;
      v_x = grad_x(x, y);
      v_y = grad_y(x, y);
      v_len = sqrt(v_x*v_x + v_y*v_y);
      g_vector(x, y, v_x / v_len, v_y / v_len, V_FACTOR);
   }}

      g_box(X_MIN, X_MAX, Y_MIN, Y_MAX, G_YES, G_NO);

      g_sleep( -1.0 );
      g_term();
}

void graphic_initialize()
{
   g_init(GRAPH_NAME, WINDOW_X_SIZE, WINDOW_Y_SIZE);
   g_device(G_DISP);
   g_def_scale(0, X_MIN, X_MAX, Y_MIN, Y_MAX,
                  X_LEFT_MM, Y_TOP_MM, X_SIZE_MM, Y_SIZE_MM);
   g_sel_scale( 0 );
   g_cls();

   g_text( SIGN_LEFT_MM, SIGN_TOP_MM, SIGN);
}

void g_vector( x_org, y_org, x_comp, y_comp, factor)
double x_org, y_org, x_comp, y_comp, factor;
{
   g_marker( x_org, y_org );
   g_move( x_org, y_org );
   g_plot( x_org+x_comp*factor, y_org+y_comp*factor);
}

double func(x, y)
double x, y;
{
   return (1.0 / ((x-1.0)*(x-1.0) + (y-1.0)*(y-1.0) + 1.0)) +
             (1.0 / ((x-7.0)*(x-7.0) + (y-8.0)*(y-8.0) + 1.0));
}

double grad_x(x, y)
double x, y;
{
   double a, b;

   a = (x-1.0)*(x-1.0) + (y-1.0)*(y-1.0) + 1.0;
   b = (x-7.0)*(x-7.0) + (y-8.0)*(y-8.0) + 1.0;
 
   return -2.0*(x-1.0) / (a*a) - 2.0*(x-7.0) / (b*b);
}

double grad_y(x, y)
double x, y;
{
   double a, b;
 
   a = (x-1.0)*(x-1.0) + (y-1.0)*(y-1.0) +1.0;
   b = (x-7.0)*(x-7.0) + (y-8.0)*(y-8.0) +1.0;

   return -2.0*(y-1.0) / (a*a) - 2*(y-8.0) / (b*b);
}
