#include <stdio.h>
#include <glsc.h>

#define GRAPH_NAME "graph20_1"
 
#define WINDOW_X_SIZE 150.0
#define WINDOW_Y_SIZE 150.0

#define SIGN "nc20_1 / t930028 / Nobuyuki Okada"
#define SIGN_LEFT_MM 5.0
#define SIGN_TOP_MM  10.0

#define X_LEFT_MM 10.0
#define Y_TOP_MM  20.0
#define X_SIZE_MM 120.0
#define Y_SIZE_MM 120.0

#define X_MAX 10.0
#define X_MIN 0.0
#define Y_MAX 10.0
#define Y_MIN 0.0

extern double f1();
extern double f2();
extern void   Runge_Kutta2();

main()
{
   int i, j;
   double a1, a2, delta_t, end_t, y1, y2;
 
   printf("a1 = ");
   scanf("%le", &a1);

   printf("a2 = ");
   scanf("%le", &a2);

   printf("delta_t = ");
   scanf("%le", &delta_t);

   printf("end_t = ");
   scanf("%le", &end_t);
  
   g_init(GRAPH_NAME, WINDOW_X_SIZE, WINDOW_Y_SIZE);
   g_device(G_DISP);
   g_def_scale(0, X_MIN, X_MAX, Y_MIN, Y_MAX, X_LEFT_MM, Y_TOP_MM,
                 X_SIZE_MM, Y_SIZE_MM);
   g_sel_scale(0);
   g_cls();

   g_text(SIGN_LEFT_MM, SIGN_TOP_MM, SIGN);

   y1 = a1;
   y2 = a2;

   g_line_color(G_RED);
   g_move(a1, a2);
   g_marker_size(2);
   g_marker_type(-1);
   g_marker(a1, a2);  
   
   j = end_t / delta_t;

   for (i = 1; i < j; i++) {
      g_move(y1, y2);
      Runge_Kutta2(end_t, delta_t, &y1, &y2);
      g_plot(y1, y2);
   }

   g_line_color(G_BLACK);
   g_move(X_MIN, Y_MIN);
   g_plot(X_MAX, Y_MIN);

   g_move(X_MIN, Y_MIN);
   g_plot(X_MIN, Y_MAX);

   g_sleep(-1.0);
   g_term();
}

double f1(t, y1, y2)
double t, y1, y2;
{
   return (2.0 - y2)*y1;
}

double f2(t, y1, y2)
double t, y1, y2;
{
   return (2.0*y1 - 3.0)*y2;
}

void Runge_Kutta2(t, delta_t, y1, y2)
double t, delta_t, *y1, *y2;
{
   double k11, k21, k31, k41;
   double k12, k22, k32, k42;

   k11 = f1(t              , *y1                  , *y2                  );
   k12 = f2(t              , *y1                  , *y2                  );
   k21 = f1(t + delta_t/2.0, *y1 + delta_t/2.0*k11, *y2 + delta_t/2.0*k12);
   k22 = f2(t + delta_t/2.0, *y1 + delta_t/2.0*k11, *y2 + delta_t/2.0*k12);
   k31 = f1(t + delta_t/2.0, *y1 + delta_t/2.0*k21, *y2 + delta_t/2.0*k22);
   k32 = f2(t + delta_t/2.0, *y1 + delta_t/2.0*k21, *y2 + delta_t/2.0*k22);
   k41 = f1(t + delta_t    , *y1 + delta_t*k31    , *y2 + delta_t*k32    );
   k42 = f2(t + delta_t    , *y1 + delta_t*k31    , *y2 + delta_t*k32    );

   *y1 = *y1 + delta_t/6.0*(k11 + 2*k21 + 2*k31 + k41);
   *y2 = *y2 + delta_t/6.0*(k12 + 2*k22 + 2*k32 + k42);
}



