#include <stdio.h>
#include <glsc.h>

#define GRAPH_NAME "graph21_2"
 
#define WINDOW_X_SIZE 150.0
#define WINDOW_Y_SIZE 150.0

#define SIGN "nc21_2 / t930028 / Nobuyuki Okada"
#define SIGN_LEFT_MM 5.0
#define SIGN_TOP_MM  10.0

#define X_LEFT_MM 10.0
#define Y_TOP_MM  20.0
#define X_SIZE_MM 120.0
#define Y_SIZE_MM 120.0

#define X_MAX 10.0
#define X_MIN 0.0
#define Y_MAX 1.0
#define Y_MIN -1.0

extern double f1();
extern double f2();
extern void   Runge_Kutta2();

double k;
double omega;

main()
{
   double t, j, a1, a2, delta_t, end_t, y1, y2;
 
   printf("k = ");
   scanf("%le", &k);

   printf("omega = ");
   scanf("%le", &omega);

   g_init(GRAPH_NAME, WINDOW_X_SIZE, WINDOW_Y_SIZE);
   g_device(G_DISP);
   g_def_scale(0, X_MIN, X_MAX, Y_MIN, Y_MAX, X_LEFT_MM, Y_TOP_MM,
                 X_SIZE_MM, Y_SIZE_MM);
   g_sel_scale(0);
   g_cls();

   g_text(SIGN_LEFT_MM, SIGN_TOP_MM, SIGN);

   a1 = 1.0;
   a2 = 0.0;
   y1 = a1;
   y2 = a2;
   delta_t = 0.1;
   end_t = 10;

   g_line_color(G_BLACK);
   g_move(0.0, 0.0);
   g_plot(X_MAX, 0.0);

   g_move(X_MIN, Y_MIN);
   g_plot(X_MIN, Y_MAX);

   g_line_color(G_RED);

   g_move(0.0, 1.0);

   for (t = 0.0; t < end_t; t=t+delta_t) {
      Runge_Kutta2(t, delta_t, &y1, &y2);
      g_plot(t, y1);
      g_move(t, y1);
   }

   g_sleep(-1.0);
   g_term();
}

double f1(t, y1, y2)
double t, y1, y2;
{
   return y2;
}

double f2(t, y1, y2)
double t, y1, y2;
{
   return -2.0*k*y2 - omega*omega*y1;
}

void Runge_Kutta2(t, delta_t, y1, y2)
double t, delta_t, *y1, *y2;
{
   double k11, k21, k31, k41;
   double k12, k22, k32, k42;

   k11 = f1(t              , *y1                  , *y2                  );
   k12 = f2(t              , *y1                  , *y2                  );
   k21 = f1(t + delta_t/2.0, *y1 + delta_t/2.0*k11, *y2 + delta_t/2.0*k12);
   k22 = f2(t + delta_t/2.0, *y1 + delta_t/2.0*k11, *y2 + delta_t/2.0*k12);
   k31 = f1(t + delta_t/2.0, *y1 + delta_t/2.0*k21, *y2 + delta_t/2.0*k22);
   k32 = f2(t + delta_t/2.0, *y1 + delta_t/2.0*k21, *y2 + delta_t/2.0*k22);
   k41 = f1(t + delta_t    , *y1 + delta_t*k31    , *y2 + delta_t*k32    );
   k42 = f2(t + delta_t    , *y1 + delta_t*k31    , *y2 + delta_t*k32    );

   *y1 = *y1 + delta_t/6.0*(k11 + 2*k21 + 2*k31 + k41);
   *y2 = *y2 + delta_t/6.0*(k12 + 2*k22 + 2*k32 + k42);
}



