#include <stdio.h>
#include <math.h>
#define MAX 100

extern void   LU_tri_decomp();
extern void   LU_tri_subst();
extern double u();

void LU_tri_decomp(N, alpha, beta, gamma, l, d)
int N;
double *alpha, *beta, *gamma, *l, *d;
{
   int i;

   d[1] = alpha[1];

   for (i = 2; i <= N; i++) {
      l[i] = beta[i]/d[i-1];
      d[i] = alpha[i] - l[i]*gamma[i-1];
   }
}

void LU_tri_subst(N, gamma, l, d, k, Y)
int N;
double *gamma, *l, *d, *k, *Y;
{
   int i;
   double z[MAX];

   z[1] = k[1];

   for (i = 2; i <= N; i++) {
      z[i] = k[i] - l[i]*z[i-1];
   }

   Y[N] = z[N]/d[N];

   for (i = N-1; i >= 1; i--) {
      Y[i] = (z[i] - gamma[i]*Y[i+1]) / d[i];
   }
}

double u(x)
double x;
{
   double t;

   t = 1 + exp(1.0) +exp(2.0);

   return (-exp(2.0)-exp(1.0))/t*exp(-x) - 1.0/t*exp(2.0*x) +1.0;
}

main()
{
   int N, i;
   double alpha[MAX], beta[MAX], gamma[MAX], l[MAX], d[MAX], k[MAX], Y[MAX];
   double D[MAX], delta_x, t, x;

   N = 20;

   delta_x = 1.0/(double)N;
 
   Y[0] = 0.0;
   Y[N] = 0.0;
  
   for (i = 1; i < N; i++) {
      alpha[i] = 2.0 + 2.0*delta_x*delta_x;
   }

   for (i = 1; i < N; i++) {
      beta[i] = -1.0 - delta_x/2.0;
   }

   for (i = 1; i < N; i++) {
      gamma[i] = -1.0 + delta_x/2.0;
   }

   for (i = 1; i < N; i++) {
      D[i] = 2.0*delta_x*delta_x;
   }

   for (i = 1; i < N; i++) {
      if (i == 1) {
         k[i] = D[i] - beta[1]*Y[0];
      }
      else if (i == N-1) {
         k[N-1] = D[N-1] - gamma[N-1]*Y[N];
      }
      else {
         k[i] = D[i];
      }
   }

   LU_tri_decomp(N-1, alpha, beta, gamma, l, d);

   LU_tri_subst(N-1, gamma, l, d, k, Y);

   for (i = 0; i <= N; i++) {
      printf("Y(%2d) = %le\n", i, Y[i]);
   }

   x = 1.0 / 4.0;
   printf("x = 1/4 | Y(05) = %le | kai = %le | gosa = %le \n",
            Y[5], u(x), fabs(u(x) - Y[5]));

   x = 1.0 / 2.0;
   printf("x = 1/2 | Y(10) = %le | kai = %le | gosa = %le \n",
            Y[10], u(x), fabs(u(x) - Y[10]));

   x = 3.0 / 4.0;
   printf("x = 3/4 | Y(15) = %le | kai = %le | gosa = %le \n",
            Y[15], u(x), fabs(u(x) - Y[15]));
}
