#include <stdio.h>
#include <glsc.h>

#define MAX 100

#define GRAPH_NAME "graph23"

#define WINDOW_X_SIZE 150.0
#define WINDOW_Y_SIZE 150.0

#define SIGN "nc23 / t930028 / Nobuyuki Okada"
#define SIGN_LEFT_MM 5.0
#define SIGN_TOP_MM  10.0

#define X_LEFT_MM 10.0
#define Y_TOP_MM  20.0
#define X_SIZE_MM 120.0
#define Y_SIZE_MM 120.0

#define X_MAX 1.0
#define X_MIN 0.0
#define Y_MAX 0.5
#define Y_MIN 0.0

extern void LU_tri_decomp();
extern void LU_tri_subst();

void LU_tri_decomp(N, alpha, beta, gamma, l, d)
int N;
double *alpha, *beta, *gamma, *l, *d;
{
   int i;

   d[1] = alpha[1];

   for (i = 2; i <= N; i++) {
      l[i] = beta[i]/d[i-1];
      d[i] = alpha[i] - l[i]*gamma[i-1];
   }
}

void LU_tri_subst(N, gamma, l, d, k, Y)
int N;
double *gamma, *l, *d, *k, *Y;
{
   int i;
   double z[MAX];

   z[1] = k[1];

   for (i = 2; i <= N; i++) {
      z[i] = k[i] - l[i]*z[i-1];
   }

   Y[N] = z[N]/d[N];

   for (i = N-1; i >= 1; i--) {
      Y[i] = (z[i] - gamma[i]*Y[i+1]) / d[i];
   }
}

main()
{
   int N, i;
   double alpha[MAX], beta[MAX], gamma[MAX], l[MAX], d[MAX], k[MAX], Y[MAX];
   double D[MAX], delta_x, j;

   g_init(GRAPH_NAME, WINDOW_X_SIZE, WINDOW_Y_SIZE);
   g_device(G_DISP);
   g_def_scale(0, X_MIN, X_MAX, Y_MIN, Y_MAX, X_LEFT_MM, Y_TOP_MM,
                 X_SIZE_MM, Y_SIZE_MM);
   g_sel_scale(0);
   g_cls();

   g_text(SIGN_LEFT_MM, SIGN_TOP_MM, SIGN);

   N = 20;

   delta_x = 1.0/(double)N;
 
   Y[0] = 0.0;
   Y[N] = 0.0;
  
   for (i = 1; i < N; i++) {
      alpha[i] = 2.0 + 2.0*delta_x*delta_x;
   }

   for (i = 1; i < N; i++) {
      beta[i] = -1.0 - delta_x/2.0;
   }

   for (i = 1; i < N; i++) {
      gamma[i] = -1.0 + delta_x/2.0;
   }

   for (i = 1; i < N; i++) {
      D[i] = 2.0*delta_x*delta_x;
   }

   for (i = 1; i < N; i++) {
      if (i == 1) {
         k[i] = D[i] - beta[1]*Y[0];
      }
      if (i == N-1) {
         k[N-1] = D[N-1] - gamma[N-1]*Y[N];
      }
      else {
         k[i] = D[i];
      }
   }

   LU_tri_decomp(N-1, alpha, beta, gamma, l, d);

   LU_tri_subst(N-1, gamma, l, d, k, Y);

   g_line_color (G_BLACK);
   g_box (X_MIN, X_MAX, Y_MIN, Y_MAX, G_YES, G_NO);

   g_line_color (G_BLUE);
   g_data_plot (X_MIN, X_MAX, Y, N+1);

   g_marker_color (G_GREEN);
   g_marker_size (3.0);
   g_marker_type (-1);
   
   for (i = 0; i <= N; i++) {
      j = i*delta_x;
      g_marker (j, Y[i]);
   }

   g_sleep(-1.0);
   g_term();
}
