#include <stdio.h>
#include <math.h>
#include <glsc.h>

#define GRAPH_NAME "graph24"

#define WINDOW_X_SIZE 150.0
#define WINDOW_Y_SIZE 150.0

#define SIGN "nc24 / t930028 / Nobuyuki Okada"
#define SIGN_LEFT_MM 5.0
#define SIGN_TOP_MM  10.0

#define X_LEFT_MM 10.0
#define Y_TOP_MM  20.0
#define X_SIZE_MM 120.0
#define Y_SIZE_MM 120.0

#define X_MAX  1.0
#define X_MIN -1.0
#define Y_MAX  1.0
#define Y_MIN -1.0

extern void   g_move3d();
extern void   g_plot3d();
extern double  f();

double nx, ny, nz;

main()
{
   int i, j;
   double delta_x, delta_y;

   fprintf( stderr, "nx ny nz = ");
   scanf("%le %le %le", &nx, &ny, &nz);

   g_init(GRAPH_NAME, WINDOW_X_SIZE, WINDOW_Y_SIZE);
   g_device(G_DISP);
   g_def_scale(0, X_MIN, X_MAX, Y_MIN, Y_MAX, X_LEFT_MM, Y_TOP_MM,
                 X_SIZE_MM, Y_SIZE_MM);
   g_sel_scale(0);
   g_cls();

   g_text(SIGN_LEFT_MM, SIGN_TOP_MM, SIGN);

   g_move3d(0.0, 0.0, 0.0);
   g_plot3d(0.0, 1.0, 0.0);
   g_plot3d(1.0, 1.0, 0.0);
   g_plot3d(1.0, 0.0, 0.0);
   g_plot3d(0.0, 0.0, 0.0);

   g_move3d(0.0, 0.0, 0.0);
   g_plot3d(0.0, 1.0, 0.0);
   g_plot3d(0.0, 1.0, 1.0);
   g_plot3d(0.0, 0.0, 1.0);
   g_plot3d(0.0, 0.0, 0.0);

   g_move3d(0.0, 0.0, 0.0);
   g_plot3d(0.0, 0.0, 1.0);
   g_plot3d(1.0, 0.0, 1.0);
   g_plot3d(1.0, 0.0, 0.0);
   g_plot3d(0.0, 0.0, 0.0);

   for (i = 0; i <= 20; i++) {
      delta_x = (double)i / 20.0;

      for (j = 0; j <= 20; j++) {
         delta_y = (double)j / 20.0;

         if (j == 0) 
            g_move3d(delta_x, delta_y, f(delta_x, delta_y));
         else
            g_plot3d(delta_x, delta_y, f(delta_x, delta_y));
      }
   }

   for (i = 0; i <= 20; i++) {
      delta_y = (double)i / 20.0;

      for (j = 0; j <= 20; j++) {
         delta_x = (double)j / 20.0;

         if (j == 0)
            g_move3d(delta_x, delta_y, f(delta_x, delta_y));
         else
            g_plot3d(delta_x, delta_y, f(delta_x, delta_y));
      }
   }

   g_sleep(-1.0);
   g_term();
}

void g_move3d(a, b, c)
double a, b, c;
{
   double s, alpha, beta, gamma, guzai, ita;

   s = -(nx*a + ny*b + nx*c) / (nx*nx + ny*ny + nz*nz);

   alpha = s*nx + a;
   beta  = s*ny + b;
   gamma = s*nz + c;

   guzai = (-ny*alpha + nx*beta) / sqrt(nx*nx + ny*ny);
   ita   = (-nx*nz*alpha - ny*nz*beta + (nx*nx + ny*ny)*gamma) /
               sqrt((nx*nx + ny*ny)*(nx*nx + ny*ny + nz*nz));

   g_move(guzai, ita);
}

void g_plot3d(a, b, c)
double a, b, c;
{
   double s, alpha, beta, gamma, guzai, ita;

   s = -(nx*a + ny*b + nx*c) / (nx*nx + ny*ny + nz*nz);

   alpha = s*nx + a;
   beta  = s*ny + b;
   gamma = s*nz + c;

   guzai = (-ny*alpha + nx*beta) / sqrt(nx*nx + ny*ny);
   ita   = (-nx*nz*alpha - ny*nz*beta + (nx*nx + ny*ny)*gamma) /
               sqrt((nx*nx + ny*ny)*(nx*nx + ny*ny + nz*nz));

   g_plot(guzai, ita);
}

double f(x, y)
double x, y;
{
   return 4.0*x*(1.0 - x)*exp(-4.0*y);
}
